/*
KDE Personal Web Server - Allow you to easily share files over http
Copyright (C) 2010 by Vitor Boschi da Silva (vitorboschi@gmail.com)

This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.

*/

#include "contentmodel.h"
#include "webcontent_virtualfolder.h"

#include "webcontent_dummy.h"
#include "webcontent_file.h"
#include <QDebug>

ContentModel::ContentModel(QObject* parent): QAbstractItemModel(parent)
{
	rootItem = new WebContent_VirtualFolder();
}

ContentModel::~ContentModel()
{
	delete(rootItem);
}

int ContentModel::columnCount(const QModelIndex& parent) const
{
	return 1;
}

QVariant ContentModel::data(const QModelIndex& index, int role) const
{
	if (!index.isValid())
		return QVariant();
	
	if (role != Qt::DisplayRole)
		return QVariant();
	
	WebContent *item = static_cast<WebContent*>(index.internalPointer());
	
	return item->getIdentifier();
}

Qt::ItemFlags ContentModel::flags(const QModelIndex& index) const
{
	if (!index.isValid())
		return 0;
	
	return Qt::ItemIsEnabled | Qt::ItemIsSelectable;
}

QVariant ContentModel::headerData(int section, Qt::Orientation orientation, int role) const
{
	if (orientation == Qt::Horizontal && role == Qt::DisplayRole && section == 0) {
    return "File";
	}
	else {
		return QVariant();
	}
}

QModelIndex ContentModel::index(int row, int column, const QModelIndex& parent) const
{
	if (!hasIndex(row, column, parent))
		return QModelIndex();
	
	WebContent *parentItem;
	
	if (!parent.isValid())
		parentItem = rootItem;
	else
		parentItem = static_cast<WebContent*>(parent.internalPointer());
	
	WebContent *childItem = parentItem->getChild(row);
	if (childItem)
		return createIndex(row, column, childItem);
	else
		return QModelIndex();
}

QModelIndex ContentModel::parent(const QModelIndex& index) const
{
	if (!index.isValid())
		return QModelIndex();
	
	WebContent *childItem = static_cast<WebContent*>(index.internalPointer());
	WebContent *parentItem = childItem->getParent();
	
	if (parentItem == rootItem)
		return QModelIndex();
	
	return createIndex(parentItem->getRow(), 0, parentItem);
}

int ContentModel::rowCount(const QModelIndex& parent) const
{
	WebContent *parentItem;
	if (parent.column() > 0) {
		return 0;
	}
	
	if (!parent.isValid()) {
		parentItem = rootItem;
	}
	else {
		parentItem = static_cast<WebContent*>(parent.internalPointer());
	}
	
	return parentItem->getChildCount();
}


QModelIndex ContentModel::addContent(WebContent* content, const QModelIndex& parent)
{
	WebContent_VirtualFolder* parentItem;
	if (!parent.isValid()) {
		parentItem = rootItem;
	}
	else {
		parentItem = static_cast<WebContent_VirtualFolder*>(parent.internalPointer());
	}
	
	//FIXME: This thing don't work everytime, because WebContent_VirtualFolder may reorder its items
	//Reseting the model as a workaround while this isn't solved
	//beginInsertRows(parent,parentItem->getChildCount(),parentItem->getChildCount());
	beginResetModel(); //
	parentItem->addContent(content);
	endResetModel();
	
	//endInsertRows();
	
	return index(parentItem->getChildCount()-1,0,parent); //return a QModelIndex pointing to the new content
}
